<?php
/*
Plugin Name: v2.60 Index Speed Test
Description: Demo of index speed differences with v2.60
Version: 1.00
Requires at least: 2.60
*/

// Internal vars, do not modify
$GLOBALS['V260IST_TABLE']       = "_v260_index_speed_test";
$GLOBALS['V260IST_FULLTABLE']   = $GLOBALS['TABLE_PREFIX'] . $GLOBALS['V260IST_TABLE'];
$GLOBALS['V260IST_STATS']   = array(); // internal use only
$GLOBALS['V260IST_MIN']   = array(); // internal use only
$GLOBALS['V260IST_MAX']   = array(); // internal use only
$GLOBALS['V260IST_TESTS'] = array(
  "Numeric Sorting"      => "SELECT * FROM `{$GLOBALS['V260IST_FULLTABLE']}` ORDER BY fourdigits LIMIT 10",
  "Numeric Match"        => "SELECT * FROM `{$GLOBALS['V260IST_FULLTABLE']}` WHERE fourdigits = 2747 ",
  "Numeric Range Search" => "SELECT COUNT(*) FROM `{$GLOBALS['V260IST_FULLTABLE']}` WHERE 450 <= fourdigits AND fourdigits <= 650",
  "Text Match"           => "SELECT * FROM `{$GLOBALS['V260IST_FULLTABLE']}` WHERE fullname = 'aliquet imperdiet' ",
  "Text Sorting"         => "SELECT * FROM `{$GLOBALS['V260IST_FULLTABLE']}` ORDER BY fullname LIMIT 10",
  "Text Starts With"     => "SELECT * FROM `{$GLOBALS['V260IST_FULLTABLE']}` WHERE fullname LIKE 'aliquet%' ",
  "Text Keyword Search"  => "SELECT * FROM `{$GLOBALS['V260IST_FULLTABLE']}` WHERE fullname LIKE '%imper%'",
);

// plugin menu - add Update Tables
pluginAction_addHandlerAndLink(t('Run Speed Test'), 'v260_index_speed_test', 'admins');
function v260_index_speed_test() {
  global $TABLE_PREFIX;
  
  print "<h1>v2.60 Index Speed Test</h1>\n";
?>


Your MySQL version: <?php print mysql_get_server_info(); ?><br/>

<?php
  
  // turn off query caching so our benchmarks have accurate numbers
  mysql_query("SET SESSION query_cache_type=0;") or die("MySQL Error: ". htmlencode(mysql_error()) . "\n");
      
  //
  _v260ist_createTestSchema(false, true); // ($useIndexes, $useDefaultColumnTypes)
  _v260ist_getTestStats(false, true);
  
  _v260ist_createTestSchema(false, false); // ($useIndexes, $useDefaultColumnTypes)
  _v260ist_getTestStats(false, false);

  _v260ist_createTestSchema(true, true); // ($useIndexes, $useDefaultColumnTypes)
  _v260ist_getTestStats(true, true);
  
  _v260ist_createTestSchema(true, false); // ($useIndexes, $useDefaultColumnTypes)
  _v260ist_getTestStats(true, false);

  _v260ist_showTestStats();
  
?><br/>

Notes:
<ul>
  <li>Each test is run 10 times to get a more accurate speed measurement.</li>
  <li>Test Table and sample data: <a href="?menu=<?php echo $GLOBALS['V260IST_TABLE'] ?>"><?php echo $GLOBALS['V260IST_TABLE'] ?></a></li>
  <li>Indexes don't help when searching for keywords in a string (only searches for keywords at the <u>beginning</u> of a string)
  <li>Custom columns used are VARCHAR(255) and INT.
</ul>

Test Descriptions: 
<ul>
<?php
  foreach ($GLOBALS['V260IST_TESTS'] as $testname => $query) {
    print "<li>" .htmlencode($testname). ": " .htmlencode($query). "</li>\n";
  }
?>
</ul>

<?php
  print "<hr/>Done, total test time: " .showExecuteSeconds(true). " seconds<br/>\n";
  
  exit;
}




function _v260ist_getTestStats($useIndexes, $useDefaultColumnTypes) {
  $prefixedTable = $GLOBALS['TABLE_PREFIX'] . $GLOBALS['V260IST_TABLE'];

  // run tests
  foreach ($GLOBALS['V260IST_TESTS'] as $name => $query) {
    // run test
    $testStart = microtime(true);
    foreach (range(1,10) as $count) { 
      $result = mysql_query($query) or die("MySQL Error: ". htmlencode(mysql_error()) . "\n");
      if (is_resource($result)) { mysql_free_result($result); }
    }
    $seconds = sprintf("%0.4f", microtime(true) - $testStart);


    
    // save result
    $GLOBALS['V260IST_STATS'][] = array(
      'testName'       => $name,
      'useIndexes'     => $useIndexes ? 1 : 0,
      'useDefaultCols' => $useDefaultColumnTypes ? 1 : 0,
      'seconds'        => $seconds,
    );
  }
}
  
//
function _v260ist_showTestStats() {

  // calculate min/max values
  foreach ($GLOBALS['V260IST_STATS'] as $testResult) {
    $GLOBALS['V260IST_MIN'][$testResult['testName']] = min( $testResult['seconds'], coalesce(@$GLOBALS['V260IST_MIN'][$testResult['testName']], $testResult['seconds']) );
    //if (!@$GLOBALS['V260IST_MIN'][$testResult['testName']]) { $GLOBALS['V260IST_MIN'][$testResult['testName']] = $testResult['seconds']; } // use first value only
    $GLOBALS['V260IST_MAX'][$testResult['testName']] = max( $testResult['seconds'], coalesce(@$GLOBALS['V260IST_MAX'][$testResult['testName']], $testResult['seconds']) ); 
  }
  
  //
  $testNames = array_unique(array_pluck($GLOBALS['V260IST_STATS'], 'testName'));

// show header
?>
<table border="1" cellspacing="0" cellpadding="2">
  <tr>
    <th>Test Name</th>
    <th>Tablename</th>
    <th>Records</th>
    <th>Auto Columns<br/>No Index</th>
    <th>Auto Columns<br/>With Index</th>
    <th>Custom Columns<br/>No Index</th>
    <th>Custom Columns<br/>With Index</th>
    <th>Speed<br/>Improvement</th>
  </tr>

  <?php foreach ($testNames as $testName): ?>
  <tr>
    <td><?php echo htmlencode($testName) ?></td>
    <td><?php echo $GLOBALS['V260IST_TABLE'] ?></td>
    <td>10,000</td>
    <?php _v260ist_showTestStats_showRowCellFor($testName, 1, 0); // testname, useDefaultCols, useIndexes ?>
    <?php _v260ist_showTestStats_showRowCellFor($testName, 1, 1); // testname, useDefaultCols, useIndexes ?>
    <?php _v260ist_showTestStats_showRowCellFor($testName, 0, 0); // testname, useDefaultCols, useIndexes ?>
    <?php _v260ist_showTestStats_showRowCellFor($testName, 0, 1); // testname, useDefaultCols, useIndexes ?>
    <td><?php echo sprintf("%0.1f", $GLOBALS['V260IST_MAX'][$testName] / $GLOBALS['V260IST_MIN'][$testName]) . 'x'; ?></td>
  </tr>  
  <?php endforeach ?>
  
</table>
<?php

}

//
function _v260ist_showTestStats_showRowCellFor($testName, $useDefaultCols, $useIndexes) {
  $testResult = array_first(array_where($GLOBALS['V260IST_STATS'], array('testName' => $testName, 'useDefaultCols' => $useDefaultCols, 'useIndexes' => $useIndexes)));
  if     ($testResult['seconds'] === $GLOBALS['V260IST_MIN'][$testResult['testName']]) { echo "<td style='background-color: #90EE90'>"; }
  elseif ($testResult['seconds'] === $GLOBALS['V260IST_MAX'][$testResult['testName']]) { echo "<td style='background-color: #EE6666'>"; }
  else { echo "<td>"; }
  echo $testResult['seconds'] . 's';
  echo "</td>"; 
}



//
function _v260ist_createTestSchema($useIndexes = false, $useDefaultColumnTypes = false) {
  //print "Re-creating test schema '{$GLOBALS['V260IST_TABLE'] }' with 10,000 records, ";
  //print $useDefaultColumnTypes ? " default column-types " : " optimized column-types ";
  //print " and "; 
  //print $useIndexes ? " indexes " : " no indexes";
  //print "<br/>\n";
  
  ### modify schema as needed
  $testSchema = loadSchema($GLOBALS['V260IST_TABLE'], dirname(__FILE__).'/pluginSchemas/'); // load schema from plugin sub-folder

  // set indexes
  $testSchema['fourdigits']['indexed'] = (bool) $useIndexes;
  $testSchema['fullname']['indexed']   = (bool) $useIndexes;
  $testSchema['title']['indexed']      = (bool) $useIndexes;

  // set col types
  $testSchema['fourdigits']['customColumnType'] = $useDefaultColumnTypes ? 'MEDIUMTEXT' : 'INT';
  $testSchema['fullname']['customColumnType']   = $useDefaultColumnTypes ? 'MEDIUMTEXT' : 'VARCHAR(255)'; // 
  $testSchema['title']['customColumnType']      = $useDefaultColumnTypes ? 'MEDIUMTEXT' : 'VARCHAR(255)'; // VARCHAR(255) or TINYTEXT
  
  // save schema to CMS /schema/ folder
  saveSchema($GLOBALS['V260IST_TABLE'], $testSchema);
  
  ### re-create mysql data
  
  // drop mysql table and  (so data, columns, and indexes get created as above)
  @mysql_query("DROP TABLE IF EXISTS `{$GLOBALS['TABLE_PREFIX']}{$GLOBALS['V260IST_TABLE']}`") or die("MySQL Error: ". htmlencode(mysql_error()) . "\n");
  createMissingSchemaTablesAndFields(); 
  
  // this is a custom backup file that doesn't DROP/CREATE table since we want to use the table we've modified above.
  $sql_data_file_path =  dirname(__FILE__)."/pluginSchemas/{$GLOBALS['V260IST_TABLE']}.defaultSqlData.php"; 
  restoreDatabase($sql_data_file_path, $GLOBALS['V260IST_TABLE']);
}



